@@ -23,7 +23,11 @@ KEY__pre_auth_key = "pre_auth_key"
 KEY__email = "email"
 KEY__sort = "sort"
 KEY__invite_key = "invite_key"
+KEY__invite_or_poll_key = "invite_or_poll_key"
+KEY__invite_poll_key = "invite_poll_key"
+KEY__invite_key_status = "invite_key_status"
 KEY__parameters = "parameters"
+KEY__oauth_token = "oauth_token"
 KEY__template = "template"
 KEY__subject = "subject"
 KEY__recipient = "recipient"
@@ -83,10 +87,10 @@ KEY__table_name = "table_name"
 KEY__is_primary = "is_primary"
 KEY__data_validation_table = "data_validation_table"
 KEY__recipient_validation_view = "recipient_validation_view"
-KEY__email_template = "email_template"
+KEY__email_template = "template"
 KEY__do_send_signup_email = "do_send_signup_email"
 KEY__allow_confirm_signup_attempt = "allow_confirm_signup_attempt"
-KEY__already_signed_up_email_template = "already_signed_up_email_template"
+KEY__already_signed_up_email_template = "existing_user_template"
 KEY__allow_signup = "allow_signup"
 KEY__email_account_name = "name"
 KEY__email_sent = "sent"
@@ -143,7 +147,6 @@ CONFIG_KEY_SERVER__port = "port"
 
 JWT_PURPOSE__oauth = "oauth"
 JWT_PURPOSE__pre_auth = "pre_auth"
-JWT_PURPOSE__invite = "invite"
 JWT_PURPOSE__connection = "connection"
 
 ERR__not_yet_installed = "JAAQL has not yet been installed!"
@@ -161,4 +164,4 @@ PORT__ems = 6061
 
 ENDPOINT__reload_accounts = "/reload-accounts"
 
-VERSION = "2.16.0"
+VERSION = "2.17.0"
