@@ -2,7 +2,7 @@ from jaaql.constants import DB__jaaql, NODE__host_node
 
 QUERY__fetch_table_columns = "SELECT column_name, is_primary FROM table_cols_marked_primary WHERE table_name = :table_name"
 QUERY__fetch_email_template = "SELECT * FROM jaaql__email_template WHERE id = :id AND deleted is NULL"
-QUERY__fetch_email_template_by_name = "SELECT * FROM jaaql__email_template WHERE name = :email_template AND deleted is NULL"
+QUERY__fetch_email_template_by_name = "SELECT * FROM jaaql__email_template WHERE name = :template AND deleted is NULL"
 QUERY__make_user_public = "UPDATE jaaql__user SET public_credentials = concat(:username, ':', :new_password), application = :application, is_public = TRUE WHERE deleted is null AND id = :user_id"
 QUERY__fetch_url_from_application_name = "SELECT url FROM jaaql__application WHERE name = :application"
 QUERY__fetch_application_public_user_credentials = "SELECT split_part(public_credentials, ':', 1) as username, split_part(public_credentials, ':', 2) as password FROM jaaql__user WHERE application = :application"
@@ -20,18 +20,18 @@ QUERY__application_sel = "SELECT * FROM jaaql__application"
 QUERY__application_count = "SELECT COUNT(*) FROM jaaql__application"
 QUERY__application_upd = "UPDATE jaaql__application SET name = coalesce(:new_name, name), description = coalesce(:new_description, description), url = coalesce(:new_url, url) WHERE name = :name"
 QUERY__database_ins = "INSERT INTO jaaql__database (node, name) VALUES (:node, :name)"
-QUERY__email_template_ins = "INSERT INTO jaaql__email_template (name, subject, account, description, app_relative_path, data_validation_table, recipient_validation_view, allow_signup, allow_confirm_signup_attempt) VALUES (:name, :subject, :account, :description, :app_relative_path, :data_validation_table, :recipient_validation_view, :allow_signup, :allow_confirm_signup_attempt)"
+QUERY__email_template_ins = "INSERT INTO jaaql__email_template (name, subject, account, description, app_relative_path, data_validation_table, data_validation_view, recipient_validation_view, allow_signup, allow_confirm_signup_attempt) VALUES (:name, :subject, :account, :description, :app_relative_path, :data_validation_table, :data_validation_view, :recipient_validation_view, :allow_signup, :allow_confirm_signup_attempt)"
 QUERY__email_account_ins = "INSERT INTO jaaql__email_account (name, send_name, protocol, host, port, username, encrypted_password) VALUES (:name, :send_name, :protocol, :host, :port, :username, :password)"
-QUERY__email_account_del = "DELETE FROM jaaql__email_account WHERE id = :id RETURNING id"
+QUERY__email_account_del = "UPDATE jaaql__email_account SET deleted = current_timestamp WHERE name = :name AND deleted is null"
 QUERY__email_account_sel = "SELECT id FROM jaaql__email_account WHERE name = :name and deleted is null"
-QUERY__email_template_del = "DELETE FROM jaaql__email_template WHERE id = :id RETURNING id"
+QUERY__email_template_del = "UPDATE jaaql__email_template SET deleted = current_timestamp WHERE name = :name and deleted is null"
 QUERY__email_template_sel = "SELECT id FROM jaaql__email_template WHERE name = :name and deleted is null"
 QUERY__email_history_sel = "SELECT * FROM jaaql__my_email_history"
 QUERY__email_history_count = "SELECT count(*) FROM jaaql__my_email_history"
 QUERY__email_history_singular_sel = "SELECT jet.name as template, jeh.sent, jeh.encrypted_subject as subject, jeh.encrypted_recipients_keys as recipient, jeh.encrypted_body as body, jeh.encrypted_attachments as attachments FROM jaaql__email_history jeh INNER JOIN jaaql__email_template jet ON jet.id = jeh.template WHERE jeh.sender = :user_id"
 QUERY__node_create = "SELECT jaaql__create_node(:name, :address, :port, :description);"
 QUERY__database_sel = "SELECT * FROM jaaql__database"
-QUERY__email_accounts_sel = "SELECT name, send_name, protocol, host, port, username FROM jaaql__email_account"
+QUERY__email_accounts_sel = "SELECT id, name, send_name, protocol, host, port, username FROM jaaql__email_account"
 QUERY__email_accounts_count = "SELECT count(*) FROM jaaql__email_account"
 QUERY__email_templates_sel = "SELECT * FROM jaaql__email_templates"
 QUERY__email_templates_count = "SELECT count(*) FROM jaaql__email_templates"
@@ -73,6 +73,11 @@ QUERY__user_ip_ins = "INSERT INTO jaaql__user_ip (the_user, address_hash, encryp
 QUERY__user_ua_ins = "INSERT INTO jaaql__user_ua (the_user, ua_hash, encrypted_ua) VALUES (:id, :ua_hash, :ua) ON CONFLICT ON CONSTRAINT jaaql__user_ua_unq DO UPDATE SET most_recent_use = current_timestamp RETURNING most_recent_use <> first_use as existed, id"
 QUERY__user_password_ins = "INSERT INTO jaaql__user_password (the_user, password_hash) VALUES (:the_user, :password_hash)"
 QUERY__fetch_user_latest_password = "SELECT id, email, password_hash, enc_totp_iv as totp_iv, last_totp, is_public FROM jaaql__user_latest_password WHERE email = lower(:username)"
+QUERY__sign_up_insert = "INSERT INTO jaaql__sign_up (the_user, email_template, data_lookup_json) VALUES (:the_user, :template, :data_lookup_json) RETURNING key_a as invite_key, key_b as invite_poll_key"
+QUERY__sign_up_poll = "SELECT key_a as invite_key, activated, the_user, closed, us.email FROM jaaql__sign_up su INNER JOIN jaaql__user us ON us.id = su.the_user WHERE (key_a = :invite_or_poll_key or key_b = :invite_or_poll_key) AND extract(epoch FROM (current_timestamp - su.created)) * 1000 < expiry_ms"
+QUERY__sign_up_fetch = "SELECT su.key_a as invite_key, su.activated, su.the_user, su.closed, su.data_lookup_json, us.email, su.email_template as template, su.data_lookup_json FROM jaaql__sign_up su INNER JOIN jaaql__user us on us.id = su.the_user WHERE ((su.key_a = :invite_or_poll_key) or (su.activated and su.key_b = :invite_or_poll_key)) AND extract(epoch FROM (current_timestamp - su.created)) * 1000 < su.expiry_ms AND su.closed is null"
+QUERY__sign_up_upd = "UPDATE jaaql__sign_up SET activated = TRUE WHERE key_a = :invite_key"
+QUERY__sign_up_close = "UPDATE jaaql__sign_up SET closed = current_timestamp WHERE key_a = :invite_key"
 QUERY__user_create_role = "SELECT jaaql__create_role(lower(:username), :password)"
 QUERY__log_ins = "INSERT INTO jaaql__log (the_user, occurred, duration_ms, encrypted_exception, encrypted_input, ip, ua, status, endpoint) VALUES (:user_id, :occurred, :duration_ms, :exception, :input, :ip, :ua, :status, :endpoint)"
 QUERY__user_log_sel = "SELECT occurred, encrypted_address as address, encrypted_ua as user_agent, status, endpoint, duration_ms, encrypted_exception as exception FROM jaaql__my_logs"
